/**
 * @file keybd.c
 * A simpletext keyboard API source
 *
 * Copyright (c) 2013, Parallax Inc.
 * Written by Steve Denson
 */
#include <stdlib.h>
#include "keybd.h"

static HUBDATA Keybd_t gkb;

static int dummyTx(keybd *ptr, int c) { return 0; }

keybd *keybd_open(int datapin, int clkpin, int locks, int autorep)
{
  /* can't use array instead of malloc because it would go out of scope. */
  text_t *text = (text_t*) malloc(sizeof(text_t));

  text->devst  = &gkb;

  text->txChar    = dummyTx;      /* required for terminal to work */
  text->rxChar    = keybd_newkey; /* required for terminal to work */

  if(locks == 0 && autorep == 0) {
    locks   = 0x04;
    autorep = 0x28;
  }

  text->cogid[0] = keybd_startx(&gkb, datapin, clkpin, locks, autorep);
  return text;
}

void keybd_close(keybd *device)
{
  int id = device->cogid[0];

  if(!device)
    return;

  if(id > 0) {
    cogstop(getStopCOGID(id));
    device->cogid[0] = 0;
  }

  free(device);
  device = 0;
}

/*
+--------------------------------------------------------------------
|  TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+------------------------------------------------------------------
*/
