
/**
 * @file keyboard.c - provides keyboard access via PASM
 *
 * Copyright (c) 2008-2013, Steve Denson
 *
 * ----------------------------------------------------------------------------
 *                        TERMS OF USE: MIT License
 * ----------------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 * ----------------------------------------------------------------------------
 */
 
#include <propeller.h>
#include "keybd.h"

/*
 * All function API are described in keyboard.h
 */

#ifndef PASMLEN
#define PASMLEN 496
#endif

int keybd_start(Keybd_t *kb, int dpin, int cpin)
{
    return keybd_startx(kb, dpin, cpin, 0x04, 0x28);
}

int keybd_startx(Keybd_t *kb, int dpin, int cpin, int locks, int autorep)
{
    extern uint32_t binary_keyboard_dat_start[];
    int id;
    kb->head = 0;
    kb->tail = 0;
    kb->present = 0;

    kb->keys[0] = dpin;
    kb->keys[1] = cpin;
    kb->keys[2] = locks;
    kb->keys[3] = autorep;

#if defined(__PROPELLER_USE_XMM__)
    // use the pasm array defined in TvText.c
    {
        extern uint32_t pasm[];
        //copy_from_xmm((uint32_t*)pasm,(uint32_t*)binary_keyboard_dat_start,PASMLEN);
        memcpy((char*)pasm,(char*)binary_keyboard_dat_start,PASMLEN<<2);
        id = cognew((uint32_t)pasm, (uint32_t)kb) + 1;
    }
#else
    id = setStopCOGID(cognew((uint32_t)binary_keyboard_dat_start, (uint32_t)kb));
#endif
    waitcnt((CLKFREQ>>4)+CNT); // wait for driver to start and configure keyboard
    return id;
}

int keybd_present(keybd *device)
{
    Keybd_t *kb = (Keybd_t*) device->devst;
    return kb->present;
}

int keybd_key(keybd *device)
{
    Keybd_t *kb = (Keybd_t*) device->devst;
    int rc = 0;
    if (kb->tail != kb->head) {
        rc = *(short*)((short*)&kb->keys[0]+kb->tail);
        kb->tail = (++kb->tail & 0xf); // move queue pointer
    }
    return rc;
}

int keybd_getkey(keybd *device)
{
    int rc = 0;
    while((rc = keybd_key(device)) == 0)
        ; // block
    return rc;
}

int keybd_newkey(keybd *device)
{
    Keybd_t *kb = (Keybd_t*) device->devst;
    kb->tail = kb->head; // block forever until key hit
    return keybd_getkey(device);
}

int keybd_gotkey(keybd *device)
{
    Keybd_t *kb = (Keybd_t*) device->devst;
    return kb->tail != kb->head;
}

void keybd_clearkeys(keybd *device)
{
    Keybd_t *kb = (Keybd_t*) device->devst;
    kb->tail = kb->head; // reset queue
}

int keybd_keystate(keybd *device, int key)
{
    Keybd_t *kb = (Keybd_t*) device->devst;
    int rc = -(kb->states[key >> 5] >> key & 1);
    return rc;
}
