/**
 * @file vgatext_tile.c
 * vgatext tile utilities
 *
 * Copyright (c) 2013, Parallax Inc
 * Written by Steve Denson
 * See end of file for terms of use.
 */

#include "vgatext.h"

/*
 * VGA_Text getTile gets tile data from x,y position
 * See header file for more details.
 */
short   vgatext_getTile(vgatext_t *vga, int x, int y)
{
    short *screen = (short*) vga->screen;
    if(x >= VGA_TEXT_COLS)
        return 0; 
    if(y >= VGA_TEXT_ROWS)
        return 0;
    return screen[y * VGA_TEXT_COLS + x];
}

/*
 * VGA_Text setTile sets tile data at x,y position
 * See header file for more details.
 */
void vgatext_setTile(vgatext_t *vga, int x, int y, short tile)
{
    short *screen = (short*) vga->screen;
    if(x >= VGA_TEXT_COLS)
        return; 
    if(y >= VGA_TEXT_ROWS)
        return;
    screen[y * VGA_TEXT_COLS + x] = tile;
}

/*
 * VGA_Text setTileColor sets tile data color at x,y position
 * See header file for more details.
 */
int vgatext_getTileColor(vgatext_t *vga, int x, int y)
{
    int shift  = 11;
    int   mask = ((VGA_TEXT_COLORS-1) << shift);
    int   ndx  = y * VGA_TEXT_COLS + x;
    int   color = 0;
    short *screen = (short*) vga->screen;
    
    if(x >= VGA_TEXT_COLS)
        return 0; 
    if(y >= VGA_TEXT_ROWS)
        return 0;
    color = screen[ndx] & mask;
    color >>= shift;
    return color;
}


/*
 * VGA_Text setTileColor sets tile data color at x,y position
 * See header file for more details.
 */
void vgatext_setTileColor(vgatext_t *vga, int x, int y, int color)
{
    short tile = 0;
    int shift  = 11;
    int   mask = ((VGA_TEXT_COLORS-1) << shift);
    int   ndx  = y * VGA_TEXT_COLS + x;
    short *screen = (short*) vga->screen;
    
    while(vga->status != VGA_TEXT_STAT_INVISIBLE)
        ;
    if(x >= VGA_TEXT_COLS)
        return; 
    if(y >= VGA_TEXT_ROWS)
        return;

    color <<= shift; 
    tile = screen[ndx];
    tile = tile & ~mask;
    tile = tile | color;
    screen[ndx] = tile;
}

/*
+--------------------------------------------------------------------
|  TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+------------------------------------------------------------------
*/

